<?php

/**
 * Piwik - Open source web analytics
 * 
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html Gpl v3 or later
 * @version $Id:  $
 */
	
/**
 * @package Piwik_Envole
 */
class Piwik_Envole extends Piwik_Plugin
{	
	public function getInformation()
	{
		$info = array(
			'name' => 'Envole',
			'description' => 'Envole // get user Info from EnvOLE',
			'author' => 'Lucile Coutouly',
			'homepage' => 'http://dev.ac-creteil.fr',
			'version' => '0.1',
			'translationAvailable' => true,
			'TrackerPlugin' => true, // this plugin must be loaded during the stats logging
		);
		return $info;
	}
	
	protected $EnvoleDb = null;
	public function __destruct()
	{
		if(!is_null($this->EnvoleDb))
		{
			Envole_close($this->EnvoleDb);
		}
	}
	
	public function install()
	{
		// add column in the visit table
		$query = "ALTER IGNORE TABLE `".Piwik::prefixTable('log_visit')."` " .
				"ADD `Envole_rne` VARCHAR( 8 ) NULL, " .
				"ADD `Envole_group` VARCHAR( 150 ) NULL, " .
				"ADD `Envole_user` VARCHAR( 4 ) NULL, " . 
				"ADD `Envole_provenance` VARCHAR( 250 ) NULL ";
				
		
		// if the column already exist do not throw error. Could be installed twice...
		try {
			Zend_Registry::get('db')->query($query);
		}
		catch(Exception $e){
		}
	}
	
	function uninstall()
	{
		$query = "ALTER TABLE `".Piwik::prefixTable('log_visit')."` " . 
			"DROP `Envole_rne`, " . 
			"DROP `Envole_provenance`, " .
			"DROP `Envole_user`, " . 
			"DROP `Envole_group`, " . 
		Zend_Registry::get('db')->query($query);
	}
	
	function getListHooksRegistered()
	{
		
		//Hook Tracker.newVisitorInformation 
		$hooks = array(
			'ArchiveProcessing_Day.compute' => 'archiveDay',
			'ArchiveProcessing_Period.compute' => 'archivePeriod',
			'Tracker.newVisitorInformation' => 'logEnvoleInfo',
			'Menu.add' => 'addMenu',
			'WidgetsList.add' => 'addWidget',
		);
		return $hooks;
	}

	function addWidget()
	{
		Piwik_AddWidget( 'Envole', 'Envole_Widgetprovenance', 'Envole', 'getEnvoleprovenance');
		Piwik_AddWidget( 'Envole', 'Envole_Widgetrne', 'Envole', 'getEnvolerne');
		Piwik_AddWidget( 'Envole', 'Envole_Widgetgroup', 'Envole', 'getEnvolegroup');
	}
	
	function addMenu()
	{
		Piwik_EditMenuUrl('General_Visitors', 'Envoleprovenance_provenance', array('module' => 'Envole'));
	}

	function archivePeriod( $notification )
	{
		$archiveProcessing = $notification->getNotificationObject();
		$dataTableToSum = array( 
				'Envole_provenance',
				'Envole_rne',
				'Envole_group',
		);
		$nameToCount = $archiveProcessing->archiveDataTable($dataTableToSum);
		$mappingFromArchiveName = array(
			'Envole_distinctProvenance' => 
						array( 	'typeCountToUse' => 'level0',
								'nameTableToUse' => 'Envole_provenance',
							),
			'Envole_distinctRne' => 
						array( 	'typeCountToUse' => 'level0',
								'nameTableToUse' => 'Envole_rne',
							),
			'Envole_distinctGroup' => 
						array( 	'typeCountToUse' => 'level0',
								'nameTableToUse' => 'Envole_group',
							)
		);
		foreach($mappingFromArchiveName as $name => $infoMapping)
		{
			$typeCountToUse = $infoMapping['typeCountToUse'];
			$nameTableToUse = $infoMapping['nameTableToUse'];
			
			$countValue = $nameToCount[$nameTableToUse]['level0'];
			$archiveProcessing->insertNumericRecord($name, $countValue);
		}
	}

	function archiveDay($notification)
	{
		$archiveProcessing = $notification->getNotificationObject();
		$this->archiveDayAggregateVisits($archiveProcessing);
		$this->archiveDayAggregateGoals($archiveProcessing);
		$this->archiveDayRecordInDatabase($archiveProcessing);
	}
	
	protected function archiveDayAggregateVisits($archiveProcessing)
	{
		// Group: for now we just consider it plain; no interaction with provenance or rne
		$labelSQL = 'Envole_group';
		$this->interestByGroup = $archiveProcessing->getArrayInterestForLabel($labelSQL);
		
		// For provenance and rne, we need to build our own stats row, with recursion 
				$query = "SELECT 	Envole_provenance, 
							Envole_rne, 
							
							count(distinct visitor_idcookie) as nb_uniq_visitors,
							count(*) as nb_visits,
							sum(visit_total_actions) as nb_actions,
							max(visit_total_actions) as max_actions, 
							sum(visit_total_time) as sum_visit_length,							
							sum(case visit_total_actions when 1 then 1 else 0 end) as bounce_count,
							sum(case visit_goal_converted when 1 then 1 else 0 end) as nb_visits_converted  
				 	FROM ".$archiveProcessing->logTable."
				 	WHERE visit_server_date = ?
				 		AND idsite = ?
				 	GROUP BY Envole_provenance, Envole_rne";
		
		$query = $archiveProcessing->db->query($query, array( $archiveProcessing->strDateStart, $archiveProcessing->idsite ));
		
		$this->interestByProvenance = 
			$this->interestByRne =
			
			$this->interestByProvenanceAndRne = array();
		
		while($row = $query->fetch() )
		{
			if(!isset($this->interestByProvenance[$row['Envole_provenance']])) $this->interestByProvenance[$row['Envole_provenance']]= $archiveProcessing->getNewInterestRow();
			if(!isset($this->interestByRne[$row['Envole_rne']])) $this->interestByRne[$row['Envole_rne']]= $archiveProcessing->getNewInterestRow();
			
			if(!isset($this->interestByProvenanceAndRne[$row['Envole_provenance']][$row['Envole_rne']])) $this->interestByProvenanceAndRne[$row['Envole_provenance']][$row['Envole_rne']]= $archiveProcessing->getNewInterestRow();
			
			$archiveProcessing->updateInterestStats( $row, $this->interestByProvenance[$row['Envole_provenance']]);
			$archiveProcessing->updateInterestStats( $row, $this->interestByRne[$row['Envole_rne']]);
			
			$archiveProcessing->updateInterestStats( $row, $this->interestByProvenanceAndRne[$row['Envole_provenance']][$row['Envole_rne']]);
		}
	}

	protected function archiveDayAggregateGoals($archiveProcessing)
	{
		// TODO
		//- add location_rne in log_conversion table
		//- run update for this plugin
		//- add icon in datatable footer
		//- add hook in tracker/goalManager to record rne as well
		//- add explanation in plugin description on how to download Envole.dat etc.
		//- on enable the plugin, check that DB is present, otherwise fail activating plugin
		/*
		$query = $archiveProcessing->queryConversionsBySegment("location_Envole_continent,Envole_provenance,Envole_rne");
		while($row = $query->fetch() )
		{
			if(!isset($this->interestByContinent[$row['location_Envole_continent']][Piwik_Archive::INDEX_GOALS][$row['idgoal']])) $this->interestByContinent[$row['location_Envole_continent']][Piwik_Archive::INDEX_GOALS][$row['idgoal']] = $archiveProcessing->getNewGoalRow();
			$archiveProcessing->updateGoalStats( $row, $this->interestByContinent[$row['location_Envole_continent']][Piwik_Archive::INDEX_GOALS][$row['idgoal']]);

			if(!isset($this->interestByprovenance[$row['Envole_provenance']][Piwik_Archive::INDEX_GOALS][$row['idgoal']])) $this->interestByprovenance[$row['Envole_provenance']][Piwik_Archive::INDEX_GOALS][$row['idgoal']] = $archiveProcessing->getNewGoalRow();
			$archiveProcessing->updateGoalStats( $row, $this->interestByprovenance[$row['Envole_provenance']][Piwik_Archive::INDEX_GOALS][$row['idgoal']]);
			
			if(!isset($this->interestByprovenanceAndrne[$row['Envole_provenance']][$row['Envole_rne']][Piwik_Archive::INDEX_GOALS][$row['idgoal']])) $this->interestByprovenanceAndrne[$row['Envole_provenance']][$row['Envole_rne']][Piwik_Archive::INDEX_GOALS][$row['idgoal']] = $archiveProcessing->getNewGoalRow();
			$archiveProcessing->updateGoalStats( $row, $this->interestByprovenanceAndrne[$row['Envole_provenance']][$row['Envole_rne']][Piwik_Archive::INDEX_GOALS][$row['idgoal']]);
		}
		$archiveProcessing->enrichConversionsByLabelArray($this->interestByContinent);
		$archiveProcessing->enrichConversionsByLabelArray($this->interestByprovenance);
		$archiveProcessing->enrichConversionsByLabelArrayHasTwoLevels($this->interestByprovenanceAndrne);
		*/
	}
	
	protected function archiveDayRecordInDatabase($archiveProcessing)
	{
		$tableProvenance = $archiveProcessing->getDataTableFromArray($this->interestByProvenance);
		$archiveProcessing->insertBlobRecord('UserEnvole_provenance', $tableProvenance->getSerialized());
		$archiveProcessing->insertNumericRecord('UserEnvole_distinctProvenance', $tableProvenance->getRowsCount());
		destroy($tableProvenance);
		
		$tableRne = $archiveProcessing->getDataTableFromArray($this->interestByRne);
		$archiveProcessing->insertBlobRecord('UserEnvole_rne', $tableRne->getSerialized());
		$archiveProcessing->insertNumericRecord('UserEnvole_distinctRne', $tableRne->getRowsCount());
		destroy($tableRne);
		
		$tableGroup = $archiveProcessing->getDataTableFromArray($this->interestByGroup);
		$archiveProcessing->insertBlobRecord('UserEnvole_group', $tableGroup->getSerialized());
		destroy($tableGroup);
	}
	
	
	public function logEnvoleInfo($notification)
	{
		/**
		/* Debuggage 
		*/
		
		/*
	  	require_once('/var/www/html/FirePHPCore/FirePHP.class.php');
		$firephp = FirePHP::getInstance(true);
		$options = array('maxObjectDepth' => 10,
                 'maxArrayDepth' => 20,
                 'useNativeJsonEncode' => true,
                 'includeLineNumbers' => true);
 
		$firephp->getOptions();
		$firephp->setOptions($options);
		$firephp->log('Plain Message');     // or FB::
		$firephp->info('Info Message');     // or FB::
		$firephp->warn('Warn Message');     // or FB::
		$firephp->error('Error Message');   // or FB::
		 
		$firephp->log('Message','Optional Label');
		 
		
		 
		$firephp->setObjectFilter('ClassName',
				           array('MemberName'));
		*/
		
		$visitorInfo =& $notification->getNotificationObject();
		
		
		try {
			$hostnameprovenance = $this->getHostnameProvenance($_SERVER["REMOTE_ADDR"]);
		
		
			$data =json_decode(html_entity_decode(Piwik_Common::getRequestVar("data")));
		
			$visitorInfo['Envole_group']=$data->group;
			$visitorInfo['Envole_rne']=$data->rne;
			$visitorInfo['Envole_user']=$data->userId;
			$visitorInfo['Envole_provenance']=$hostnameprovenance;
			
		}
		catch(Exception $e){
			
		}
		
	}

	
	
	
	public function footerUserprovenance($notification)
	{
		$out =& $notification->getNotificationObject();
		$out .= '<h2>Envole provenance</h2>';
		$out .= Piwik_FrontController::getInstance()->fetchDispatch('Envole','getEnvoleprovenance');
		$out .= '<h2>Envole Continent</h2>';
		$out .= Piwik_FrontController::getInstance()->fetchDispatch('Envole','getEnvoleContinent');
	}
	
	private function getHostnameProvenance($hostname)
	{
		
			$e = explode('.', $hostname);
			$s = sizeof($e);
			
			// if extension not correct
			if(($e[0]=="172"&&$e[1]=="30")||($e[0]=="172"&&$e[1]=="16")||($e[0]=="10"&&($e[1]=="77"||$e[1]=="93"||$e[1]=="94")))
			{
				return 'interne';
			}
			else
			{
				return 'externe';
			}
		
	}

	
}
